package main

import (
	"runtime/pprof"
	"os"
	"flag"
//	"time"
	"log"
	"ORsearch/workers"
	"ORsearch/parsedata"
	"ORsearch/structures"
)

func main() {
	//Channel to signal free thread availability
	var freethread = make(chan structures.Empty)
	//Channel to communicate best results of a thread
	var results = make(chan structures.SearchResult)
	
	var inputf string
	var outputf string
	flag.StringVar(&inputf, "skip", "", "File name of branches to skip as being already checked.")
	flag.StringVar(&outputf, "log", "distributedsearch.log", "Log file name for branches that are checked.")
	var cpuprofile = flag.String("cpuprofile", "", "write cpu profile to file")
	
	flag.Parse()

	// Build a list of all the already search branches from "inputf"
	// This is to allow interupting the code without having to recheck the already checked branches
	branchesdone := parsedata.ReadOldBranches(inputf)
	
	// Profiling stuff
	if *cpuprofile != "" {
		f, err := os.Create(*cpuprofile)
		if err != nil {
			log.Fatal(err)
		}
		pprof.StartCPUProfile(f)
		defer pprof.StopCPUProfile()
	}

	go func () {
		// Start a worker for each branch
		workers.StartWorkers(freethread, results, branchesdone) 
		// Tell GatherResults that we are done
		close(results)
	}()

	// Parse results of finished workers
	parsedata.GatherResults(results, outputf) 

}
