package parsedata

import (
	"fmt"
	"os"
	"log"
	"encoding/json"
	"ORsearch/consts"
	"ORsearch/structures"
)



// Output best matrices to standard output, branches checked to output file
func GatherResults(res chan structures.SearchResult, outputf string) {

	// Set up output device
	// One might want to add the O_SYNC property to make sure checked branches get recorded right away, so that none of them are lost for sure.
	// Beware that this might slow down the process in case of a lot of small branches (resulting in a large number of synchronous write request to the OS).
	outfile, err := os.OpenFile(outputf, os.O_WRONLY | os.O_CREATE | os.O_TRUNC, 0644) // For write access, synchronized
	if err != nil {
		log.Fatal(err)
	}
	defer outfile.Close()

	jsonout := json.NewEncoder(outfile)
	maxlength := 0
	
	// Parse incoming results
	for newresult := range res {
		
		// Update best matrix
		if newresult.Bestlength > maxlength {
		 	maxlength = newresult.Bestlength
		 	if consts.DISPLAY >= 0 {
		 		fmt.Println("New record:", maxlength)
		 		newresult.Bestmatrix.Display()
		 	}
		}
		// Record startlevels checked
		jsonout.Encode(newresult.Startlevels)


	}

}