package structures

import (
	"ORsearch/consts"
	"ORsearch/utilities"
)



// *** CREATION AND INITIALIZATION ***



// The maximum expected number of rows M (according to Hansen & Zwick's conjecture) for N columns
func ConfigureM(N int) int {
	return utilities.Fibonacci(N+2)+1
}

// The potential first rows of m are defined here. Parameter rows simply tels us how many rows to take.
// The first and second rows can be set to all zeros and all ones respectively WLOG so rows should be at least 2
// For all N <= 7, the 4 first rows of the extremal matrices always have the same structure so rows = 4 is reasonable
func InitialRows(rows int) []uint16 {
	var Init = make([]uint16, 4)

	Init[0] = 0
	Init[1] = 1<<uint(consts.N)-1
	Init[2] = 1
	Init[3] = 1<<uint(consts.N-1)-1

	return Init[:rows]
}

// Create the matrix and fill its first INITIALLEVEL+1 rows from Init
func InitializeMatrix(Init []uint16) *Matrix{
	var m = NewMatrix(ConfigureM(consts.N), consts.N)
	// We know the first Init rows of m for sure
	for i := range Init {
		m.Elems[i] = Init[i]
	}
	return m
}